import React, { FC, useState, useRef, ReactNode, isValidElement, Children, useCallback, useEffect } from 'react';
import styled from 'styled-components';

interface TabsProps {
  children: ReactNode;
}

interface PanelProps {
  label: string;
  children: ReactNode;
}

const TabPanel = styled.div`
  &[aria-hidden='false'] {
    display: block;
  }

  &[aria-hidden='true'] {
    display: none;
  }
`;

const TabButton = styled.button`
  color: #333333;
  background-color: transparent;
  border: none;
  padding: 10px 8px;

  &[aria-selected='true'] {
    color: #1861d8;
    background-color: #f2f2f5;
    border-bottom: 3px solid #1861d8;
  }
`;

export const AdminTabsPanel: FC<PanelProps> = (props: PanelProps) => {
  const { label, children } = props;
  return (
    <div
      style={{
        height: '100%',
        boxSizing: 'border-box',
      }}
    >
      <h4
        style={{
          position: 'fixed',
          top: 0,
          left: 0,
          width: '4px',
          height: '4px',
          opacity: 0,
          overflow: 'hidden',
          border: 'none',
          margin: 0,
          padding: 0,
          display: 'block',
          visibility: 'visible',
        }}
      >
        {label}
      </h4>
      {children}
    </div>
  );
};

export const AdminTabs: FC<TabsProps> = (props: TabsProps) => {
  const { children } = props;
  const [activeTabIndex, setActiveTabIndex] = useState(0);
  const tabRefs = useRef<HTMLButtonElement[]>([]);

  const setActiveTabHandler = (index: React.SetStateAction<number>) => {
    setActiveTabIndex(index);
  };

  const handleTabListKeyDown = useCallback(
    (e: React.KeyboardEvent<HTMLDivElement>) => {
      if (e.key === 'ArrowRight' || e.key === 'ArrowLeft') {
        const tabs = Children.toArray(children);
        if (tabs.length > 0) {
          if (activeTabIndex >= 0) {
            if (e.key === 'ArrowRight') {
              const nextIndex = activeTabIndex + 1 < tabs.length ? activeTabIndex + 1 : 0;
              setActiveTabIndex(nextIndex);
            } else {
              const nextIndex = activeTabIndex - 1 >= 0 ? activeTabIndex - 1 : tabs.length - 1;
              setActiveTabIndex(nextIndex);
            }
          }
        }
      }
    },
    [children, activeTabIndex]
  );

  useEffect(() => {
    if (tabRefs.current[activeTabIndex]) {
      tabRefs.current[activeTabIndex].focus();
    }
  }, [activeTabIndex]);

  return (
    <div>
      {/* eslint-disable-next-line jsx-a11y/interactive-supports-focus */}
      <div className="acms-admin-tabs-header" role="tablist" onKeyDown={handleTabListKeyDown}>
        {Children.map(children, (child, index) => {
          if (isValidElement(child)) {
            return (
              <TabButton
                ref={(node: HTMLButtonElement) => {
                  tabRefs.current[index] = node;
                }}
                role="tab"
                aria-controls={child.props.label}
                aria-selected={index === activeTabIndex}
                tabIndex={index === activeTabIndex ? 0 : -1}
                type="button"
                onClick={() => setActiveTabHandler(index)}
                key={child.props.label}
              >
                {child.props.label}
              </TabButton>
            );
          }
          return null;
        })}
      </div>
      <div
        className="acms-admin-tabs"
        style={{
          height: 'auto',
          display: 'grid',
          width: '100%',
        }}
      >
        {Children.map(children, (child, index) => {
          if (isValidElement(child)) {
            return (
              <TabPanel
                role="tabpanel"
                aria-hidden={activeTabIndex !== index}
                className="acms-admin-tabs-panel"
                key={child.props.label}
                tabIndex={0}
              >
                {child}
              </TabPanel>
            );
          }
          return null;
        })}
      </div>
    </div>
  );
};
